package util;

import java.util.Map;
import java.util.Random;
import java.util.concurrent.*;

public class OTPService {

    private static final int EXPIRE_MINS = 5;
    private final Map<String, Integer> otpCache;
    private final ScheduledExecutorService scheduler;

    private static OTPService instance;

    public static OTPService getInstance() {
        if (instance == null) {
            instance = new OTPService();
        }
        return instance;
    }

    public OTPService() {
        otpCache = new ConcurrentHashMap<>();
        scheduler = Executors.newSingleThreadScheduledExecutor();
    }

    public int generateOTP(String key) {
        Random random = new Random();
        int otp = 100000 + random.nextInt(900000);
        otpCache.put(key, otp);

        // Schedule to remove the OTP after the expiration time
        scheduler.schedule(() -> otpCache.remove(key), EXPIRE_MINS, TimeUnit.MINUTES);

        return otp;
    }

    public int getOtp(String key) {
        return otpCache.getOrDefault(key, 0);
    }

    public void clearOTP(String key) {
        otpCache.remove(key);
    }

    public String sendOTP(String email) {
        try {
            int otp = generateOTP(email);
            return sendOTPMail(email, String.valueOf(otp));
        } catch (Exception e) {
            e.printStackTrace();
            return "fail: Email Not Sent!";
        }
    }

    public boolean validate(String username, int otp) {
        int serverOtp = getOtp(username);
        if (serverOtp > 0 && otp == serverOtp) {
            clearOTP(username);
            return true;
        }
        return false;
    }

    private String sendOTPMail(String email, String otp) {
        // Implement a basic email sending function using JavaMail API or any other method
        // Here, we just return a success message for simplicity
        System.out.println("Sending OTP " + otp + " to email: " + email);
        return "success: OTP Sent!";
    }

    public static void main(String[] args) {
        OTPService otpService = new OTPService();

        // Example usage
        String email = "user@example.com";
        String result = otpService.sendOTP(email);
        System.out.println(result);

        int otp = otpService.getOtp(email);
        System.out.println("Generated OTP: " + otp);

        boolean isValid = otpService.validate(email, otp);
        System.out.println("Is OTP valid? " + isValid);
    }
}
